#include <stdio.h>
#include "board_diag.h"
#include "SD_Card.H"

#include <io.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "system.h"

void			iow(unsigned int reg, unsigned int data);
unsigned int	ior(unsigned int reg);

unsigned int nios_pio_spi_transfer(unsigned char*, unsigned char);
void nios2_pio_spi_set_init_clk(void);
void nios2_pio_spi_set_max_clk(void);
void nio2_pio_spi_unselect(void);
void nios2_pio_spi_select(void);
void nios2_pio_spi_init(void);

//-----------------------------------------------------------------------------
// Basic write access to SD card interface: writes 8 bits of "data" to register "reg".
void iow(unsigned int reg, unsigned int data)
{
	IOWR_8DIRECT(SD_CARD_BASE, reg , data);
}

//-----------------------------------------------------------------------------
// Basic read access to SD card interface: reads 8 bits from register "reg".
unsigned int ior(unsigned int reg)
{
	return IORD_8DIRECT(SD_CARD_BASE, reg);
}

//-----------------------------------------------------------------------------
// reset (init) the spi interface (f=400kHz, /CS='1', tx/rx register clear)
void nios2_pio_spi_init(void)
{
    iow(SD_cmd_reg, 0x08);
}

//-----------------------------------------------------------------------------
// set spi clock to 400kHz needed by initialisation process
void nios2_pio_spi_set_init_clk(void)
{
    iow(SD_cmd_reg, 0xFD & ior(SD_cmd_reg));
}

//-----------------------------------------------------------------------------
// set spi clock to 25MHz for regular data transfers
void nios2_pio_spi_set_max_clk(void)
{
    iow(SD_cmd_reg, 0x02 | ior(SD_cmd_reg));
}

//-----------------------------------------------------------------------------
// de-select SD-card
void nio2_pio_spi_unselect(void)
{
    iow(SD_cmd_reg, 0xEF & ior(SD_cmd_reg));
}

//-----------------------------------------------------------------------------
// select SD-Card
void nios2_pio_spi_select(void)
{
    iow(SD_cmd_reg, 0x10 | ior(SD_cmd_reg));
}

//-----------------------------------------------------------------------------
// write and read one byte
unsigned int nios_pio_spi_transfer(unsigned char* in, unsigned char out)
{
    // write data in transmit register
    iow(SD_data_reg, out);

    // wait until data is transmitted
    while(ior(SD_cmd_reg) & 0x04);

    // write dummy data in transmit register
    iow(SD_data_reg, 0x00);

    // wait until data is received
    while(ior(SD_cmd_reg) & 0x04);

    // return the content of receiver buffer.
    return(ior(SD_data_reg));
}